﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.AlgorithmBehavior.Analyzers {
  [Item("CombinedOperator", "An operator that executes a collection of other operators.")]
  [StorableClass]
  public class CombinedOperator : MultiOperator<IOperator> {
    private ScopeParameter CurrentScopeParameter {
      get { return (ScopeParameter)Parameters["CurrentScope"]; }
    }
    public IScope CurrentScope {
      get { return CurrentScopeParameter.ActualValue; }
    }

    [StorableConstructor]
    protected CombinedOperator(bool deserializing) : base(deserializing) { }
    protected CombinedOperator(CombinedOperator original, Cloner cloner)
      : base(original, cloner) {
    }

    public CombinedOperator()
      : base() {
      Parameters.Add(new ScopeParameter("CurrentScope", "The current scope whose variables and sub-scopes should be removed."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new CombinedOperator(this, cloner);
    }

    public virtual void InitializeOperators() { }

    protected IOperation BaseApply() {
      return base.Apply();
    }

    public override IOperation Apply() {
      if (Operators.Count == 0)
        InitializeOperators();

      OperationCollection next = new OperationCollection(base.Apply());
      OperationCollection inner = new OperationCollection();
      inner.Parallel = false;

      for (int i = 0; i < Operators.Count; i++) {
        inner.Add(ExecutionContext.CreateOperation(Operators[i], CurrentScope));
      }
      next.Insert(0, inner);
      return next;
    }
  }
}
